import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:spark_form/spark.dart';
import 'package:spark_form/spark_verification.dart';
import 'package:spark_form/spark_widgets.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatefulWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  FormManage? _manageData;

  FormManage get _manage => _manageData!;

  final Map<String, ManageFieldItem> _fieldMap = {
    'test': ManageFieldItem(data: '11'),
    'test1': ManageFieldItem(data: ''),
    'test2': ManageFieldItem(
        data: SparkLabelTitleData.from('hahshdh', 'change value')),
    'test3': ManageFieldItem(
        data: SparkLabelTitleData.from('hahshdh', 'change value')),
    'test4': ManageFieldItem(
        data: SparkLabelTitleData.from('hahshdh', 'change value')),
    'test5': ManageFieldItem(data: '2'),
    'test6': ManageFieldItem(data: '2'),
    'test7': ManageFieldItem(data: '2'),
    'select': ManageFieldItem(
      data: List.generate(
        5,
        (index) => SparkTagGroupItem.from(
          name: 'hh$index a',
          data: '$index',
        ),
      ),
    ),
    'select1': ManageFieldItem(
      data: List.generate(
        5,
        (index) => SparkTagGroupItem.from(
          name: 'hh$index a',
          data: '$index',
        ),
      ),
    ),
    'select2': ManageFieldItem(
      data: List.generate(
        5,
        (index) => SparkTagGroupItem.from(
          name: 'hh$index a',
          data: '$index',
        ),
      ),
    ),
  };

  final FormController _controller = FormController();

  void check() {
    _controller.validate();
  }

  void _handleSubmit(BuildContext context) {
    var result = _controller.validate(focusError: true);
    var data = _manage.getJson();
    print('get all data: $data');
    String resultData =
        'get all data:\n\n ${jsonEncode(data)}\n\n\n check result: ${result.success} field: ${result.fieldName} error: ${result.errorMessage}';
    showModalBottomSheet(
      context: context,
      builder: (_) => Center(
        child: Text(
          resultData,
          style: !result.success ? const TextStyle(color: Colors.red) : null,
        ),
      ),
    );
    // _manage.reset();
    // print('reset form data');
  }

  @override
  void initState() {
    _manageData = FormManage.create(_fieldMap);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    double _labelWidth = 60.0;
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          backgroundColor: const Color(0xFF2878FF),
          title: const Text('Spark form demo'),
          elevation: 0,
        ),
        body: Column(
          children: [
            SingleChildScrollView(
              padding: const EdgeInsets.symmetric(horizontal: 14),
              child: SparkForm(
                controller: _controller,
                manage: _manage,
                formBuilder: (BuildContext context) {
                  return Column(
                    children: [
                      SparkFormItem(
                        rules: [
                          SparkLengthVerification.length(
                            10,
                            errorMessage: 'length max',
                            type: VerifyType.change,
                            strategy: FormStrategy.limit,
                          ),
                          SparkFormatVerification.number(
                            errorMessage: 'only support number',
                            type: VerifyType.change,
                            strategy: FormStrategy.tip,
                          ),
                          SparkRequiredVerification.required(
                            errorMessage: 'is not empty',
                            type: VerifyType.change,
                            strategy: FormStrategy.tip,
                          ),
                        ],
                        spaceBetweenLayout: false,
                        label: 'input',
                        fieldName: 'test',
                        child: SparkTextField(
                          manage: _manage['test'],
                          maxLines: 1,
                          cleanble: true,
                          textDirection: TextDirection.rtl,
                          hitText: 'please input some',
                          contentPadding:
                              const EdgeInsets.symmetric(horizontal: 10),
                        ),
                      ),
                      SparkFormItem(
                        readOnly: true,
                        label: 'select',
                        isRequired: true,
                        padding: const EdgeInsets.symmetric(
                            horizontal: 0, vertical: 10),
                        rules: [
                          SparkRequiredVerification.required(
                            errorMessage: 'pick one',
                            type: VerifyType.change,
                            strategy: FormStrategy.tip,
                          ),
                        ],
                        fieldName: 'test1',
                        child: SparkSelect(
                          manage: _manage['test1'],
                          options: [
                            SparkSelectOption.form('sd', 'hahah'),
                            SparkSelectOption.form('gh', 'hasdahah')
                          ],
                        ),
                      ),
                      SparkFormItem(
                        fieldName: 'test2',
                        label: 'title',
                        readOnly: true,
                        child: SparkLabelTitle(
                          manage: _manage['test2'],
                          contentPadding: const EdgeInsets.only(right: 10),
                          onTap: (_data, _updater) {
                            _data.title = 'hhhhhhhh';
                            _data.data = 'what\'s up';
                            _updater();
                          },
                        ),
                      ),
                      Row(
                        mainAxisSize: MainAxisSize.max,
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: [
                          ConstrainedBox(
                            child: SparkFormItem(
                              labelWidth: _labelWidth,
                              fieldName: 'test3',
                              label: 'titleRow',
                              readOnly: true,
                              child: SparkLabelTitle(
                                manage: _manage['test3'],
                                selectDown: true,
                                contentPadding:
                                    const EdgeInsets.only(right: 10),
                                padding: const EdgeInsets.only(right: 5),
                                onTap: (_data, _updater) {
                                  _data.title =
                                      'test test test test test test test test test test test test test test';
                                  _data.data = 'what\'s up';
                                  _updater();
                                },
                                width: (MediaQuery.of(context).size.width -
                                            _labelWidth * 2 -
                                            92) /
                                        2 -
                                    14,
                              ),
                            ),
                            constraints: BoxConstraints(
                                maxWidth:
                                    MediaQuery.of(context).size.width * 0.66 -
                                        14),
                          ),
                          Container(
                            width: 1,
                            height: 14,
                            color: const Color(0xFFcccccc),
                          ),
                          ConstrainedBox(
                            child: SparkFormItem(
                              labelWidth: 0,
                              fieldName: 'test4',
                              // label: 'titleRow',
                              labelBuilder: (_, _t) => Container(),
                              readOnly: true,
                              spaceBetweenLayout: true,
                              child: SparkLabelTitle(
                                manage: _manage['test4'],
                                contentPadding:
                                    const EdgeInsets.only(right: 10),
                                selectDown: true,
                                onTap: (_data, _updater) {
                                  _data.title = 'hanbsdhjb';
                                  _data.data = 'what\'s up';
                                  _updater();
                                },
                              ),
                            ),
                            constraints: BoxConstraints(
                                maxWidth:
                                    MediaQuery.of(context).size.width * .33 -
                                        14),
                          ),
                        ],
                      ),
                      SparkFormItem(
                        fieldName: 'select',
                        label: 'group',
                        readOnly: true,
                        crossCenter: false,
                        isRequired: true,
                        spaceBetweenLayout: false,
                        child: Row(
                          crossAxisAlignment: CrossAxisAlignment.baseline,
                          textBaseline: TextBaseline.ideographic,
                          children: [
                            Expanded(
                              child: SparkTagGroup(
                                manage: _manage['select'],
                                type: SparkTagGroupType.multiple,
                                placeholder: 'you can add option',
                                showClean: true,
                              ),
                              flex: 1,
                            ),
                            Container(
                              width: 1,
                              height: 14,
                              color: const Color(0xFFcccccc),
                              margin:
                                  const EdgeInsets.symmetric(horizontal: 10),
                            ),
                            GestureDetector(
                              onTap: () {
                                (_manage['select'].notifier.data
                                        as List<SparkTagGroupItem>)
                                    .add(SparkTagGroupItem.from(name: 'test'));
                                _manage['select'].update();
                              },
                              child: const Icon(
                                Icons.search,
                                size: 18,
                              ),
                            ),
                          ],
                        ),
                      ),
                      SparkFormItem(
                        fieldName: 'select1',
                        label: 'mult',
                        readOnly: true,
                        crossCenter: false,
                        isRequired: true,
                        rules: [
                          SparkRequiredVerification.required(
                            errorMessage: 'pick one',
                            type: VerifyType.all,
                            strategy: FormStrategy.tip,
                          ),
                        ],
                        spaceBetweenLayout: false,
                        child: SparkTagGroup(
                          manage: _manage['select1'],
                          type: SparkTagGroupType.multipleAndIncrease,
                          placeholder: 'add option',
                          showClean: true,
                        ),
                      ),
                      SparkFormItem(
                        fieldName: 'select2',
                        label: 'singe',
                        readOnly: true,
                        crossCenter: false,
                        isRequired: true,
                        spaceBetweenLayout: false,
                        child: SparkTagGroup(
                          manage: _manage['select2'],
                          type: SparkTagGroupType.singe,
                          placeholder: 'add option',
                        ),
                      ),
                    ],
                  );
                },
              ),
            ),
            Builder(
                builder: (_) => GestureDetector(
                      onTap: () {
                        _handleSubmit(_);
                      },
                      child: Container(
                        margin: const EdgeInsets.only(top: 30),
                        width: MediaQuery.of(_).size.width - 40,
                        height: 50,
                        alignment: Alignment.center,
                        decoration: const BoxDecoration(
                          borderRadius: BorderRadius.all(Radius.circular(40)),
                          color: Color(0xFF2878FF),
                        ),
                        child: const Text(
                          'Submit',
                          style: TextStyle(color: Colors.white, fontSize: 18.0),
                        ),
                      ),
                    ))
          ],
        ),
      ),
    );
  }
}
