part of spark_widget;

class SparkLabelTitle extends SparkFormWidget {
  const SparkLabelTitle({
    Key? key,
    required FormItemManage manage,
    this.suffixIcon,
    this.showSuffix,
    this.suffixSize,
    this.maxLines,
    this.onTap,
    this.hitText,
    this.style,
    this.hitTextStyle,
    this.padding = EdgeInsets.zero,
    this.margin,
    this.width,
    this.selectDown = false,
    this.contentPadding,
  }) : super(key: key, manage: manage);

  final Widget? suffixIcon;

  final bool? showSuffix;

  final Size? suffixSize;

  final int? maxLines;

  final Function(SparkLabelTitleData data, Updater updater)? onTap;

  final String? hitText;

  final TextStyle? style;

  final TextStyle? hitTextStyle;

  /// set body padding
  final EdgeInsets padding;

  final EdgeInsets? margin;

  final double? width;

  final bool selectDown;

  /// set content and suffix padding
  final EdgeInsets? contentPadding;

  @override
  State<StatefulWidget> createState() => _SparkLabelTitleBase();
}

class _SparkLabelTitleBase
    extends SparkFormWidgetBase<SparkLabelTitle, SparkLabelTitleData> {
  void _clickHandle() {
    if (widget.onTap != null) {
      widget.onTap!(fieldData, () {
        manage.update();
      });
    }
  }

  @override
  initState() {
    manage.registerClickCallback(_clickHandle);
    super.initState();
  }

  Widget _buildLabelTitle(BuildContext context, SparkTheme? theme) {
    String _title = fieldData.title;
    bool isHitText = false;
    if (emptyOrNullString(_title)) {
      _title = widget.hitText ?? '';
      isHitText = true;
    }
    TextStyle? _style = !isHitText
        ? (widget.style ??
            TextStyle(
              fontSize: theme?.labelFontSize,
              color: theme?.labelColor,
            ))
        : (widget.hitTextStyle ??
            TextStyle(
              fontSize: theme?.labelFontSize,
              color: theme?.hitColor,
            ));
    return Expanded(
      flex: 1,
      child: Container(
        alignment: Alignment.topRight,
        padding: widget.contentPadding,
        margin: widget.margin,
        width: widget.width,
        child: Text(
          _title,
          style: _style,
          maxLines: widget.maxLines,
          overflow: TextOverflow.ellipsis,
        ),
      ),
    );
  }

  Widget? _buildSuffix() {
    if (widget.showSuffix ?? false) {
      return null;
    }
    Size _size = widget.suffixSize ?? const Size(12, 12);
    return SizedBox(
      width: _size.width,
      height: _size.height,
      child: widget.suffixIcon ?? _buildDefaultSuffixIcon(_size),
    );
  }

  Widget _buildDefaultSuffixIcon(Size size) {
    return Image.asset(
      widget.selectDown
          ? 'icons/icon_arrow_down.png'
          : 'icons/icon_arrow_right_grey.png',
      width: size.width,
      height: size.height,
      scale: 1,
      package: 'spark_form',
    );
  }

  @override
  dispose() {
    // just remove
    manage.clearClickCallback();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    Widget? _suffix = _buildSuffix();
    SparkTheme? theme = SparkThemeData.of(context);
    return SparkWrapper(
      manage: manage,
      builder: (_, _value, _updater) {
        return Padding(
          padding: widget.padding,
          child: Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.end,
            children: [
              _buildLabelTitle(context, theme),
              if (_suffix != null) _suffix,
            ],
          ),
        );
      },
    );
  }

  @override
  void valueChangeNotify() {
    //
  }
}

class SparkLabelTitleData extends SparkFormDataChangeImp {
  SparkLabelTitleData(this.title, dynamic data) : super(data);

  SparkLabelTitleData.from(this.title, dynamic data) : super(data);

  String title;
}
